@@ -27,7 +27,7 @@
  * list of people on the gedit Team.
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-commands-file.c 5462 2007-01-27 17:10:16Z pborelli $
+ * $Id: gedit-commands-file.c 5663 2007-06-29 08:52:15Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -385,6 +385,9 @@ open_dialog_response_cb (GeditFileChooserDialog *dialog,
 
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 
+	/* Remember the folder we navigated to */
+	 _gedit_window_set_default_path (window, uris->data);
+
 	gedit_commands_load_uris (window,
 				  uris,
 				  encoding,
@@ -416,7 +419,7 @@ _gedit_cmd_file_open (GtkAction   *action,
 		return;
 	}
 
-	open_dialog = gedit_file_chooser_dialog_new (_("Open Files\342\200\246"),
+	open_dialog = gedit_file_chooser_dialog_new (_("Open Files"),
 						     GTK_WINDOW (window),
 						     GTK_FILE_CHOOSER_ACTION_OPEN,
 						     NULL,
@@ -525,7 +528,7 @@ open_location_dialog_response_cb (GeditOpenLocationDialog *dlg,
 					      _("The entered location is not valid."));
 
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (msg),
-							  _("Please, check that you typed the "
+							  _("Please check that you typed the "
 							    "location correctly and try again."));
 
 		gtk_window_group_add_window (wg, GTK_WINDOW (msg));
@@ -733,6 +736,10 @@ save_dialog_response_cb (GeditFileChooserDialog *dialog,
 
 		g_free (uri_for_display);
 
+		/* let's remember the dir we navigated too,
+		 * even if the saving fails... */
+		 _gedit_window_set_default_path (window, uri);
+
 		_gedit_tab_save_as (tab, uri, encoding);
 	}
 
