@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-document.c,v 1.136 2006/11/08 15:15:54 sfre Exp $
+ * $Id: gedit-document.c 5662 2007-06-28 23:26:53Z sfre $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -39,20 +39,21 @@
 
 #include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs.h>
+#include <gtksourceview/gtksourceiter.h>
 
 #include "gedit-prefs-manager-app.h"
 #include "gedit-document.h"
 #include "gedit-debug.h"
 #include "gedit-utils.h"
 #include "gedit-metadata-manager.h"
-#include "gedit-languages-manager.h"
+#include "gedit-language-manager.h"
+#include "gedit-source-style-manager.h"
 #include "gedit-document-loader.h"
 #include "gedit-document-saver.h"
 #include "gedit-marshal.h"
+#include "gedit-enum-types.h"
 #include "gedittextregion.h"
 
-#include <gtksourceview/gtksourceiter.h>
-
 #undef ENABLE_PROFILE 
 
 #ifdef ENABLE_PROFILE
@@ -73,6 +74,10 @@ PROFILE (static GTimer *timer = NULL)
 
 #define GEDIT_DOCUMENT_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), GEDIT_TYPE_DOCUMENT, GeditDocumentPrivate))
 
+static void	gedit_document_save_real	(GeditDocument          *doc,
+						 const gchar            *uri,
+						 const GeditEncoding    *encoding,
+						 GeditDocumentSaveFlags  flags);
 static void	gedit_document_set_readonly	(GeditDocument *doc,
 						 gboolean       readonly);
 static void	to_search_region_range 		(GeditDocument *doc,
@@ -92,7 +97,6 @@ struct _GeditDocumentPrivate
 	gint	     readonly : 1;
 	gint	     last_save_was_manually : 1; 	
 	gint	     language_set_by_user : 1;
-	gint         is_saving_as : 1;
 	gint         stop_cursor_moved_emission : 1;
 
 	gchar	    *uri;
@@ -143,6 +147,7 @@ enum {
 	CURSOR_MOVED,
 	LOADING,
 	LOADED,
+	SAVE,
 	SAVING,
 	SAVED,
 	SEARCH_HIGHLIGHT_UPDATED,
@@ -216,8 +221,6 @@ gedit_document_finalize (GObject *object)
 	{
 		GtkTextIter iter;
 		gchar *position;
-		gchar *lang_id = NULL;
-		GtkSourceLanguage *lang;
 
 		gtk_text_buffer_get_iter_at_mark (
 				GTK_TEXT_BUFFER (doc),			
@@ -234,15 +237,13 @@ gedit_document_finalize (GObject *object)
 
 		if (doc->priv->language_set_by_user)
 		{
+			GtkSourceLanguage *lang;
+
 			lang = gedit_document_get_language (doc);
 
-			if (lang != NULL)
-				lang_id = gtk_source_language_get_id (lang);
-
 			gedit_metadata_manager_set (doc->priv->uri,
-						    "language",
-						    (lang_id == NULL) ? "_NORMAL_" : lang_id);
-			g_free (lang_id);
+				    "language",
+				    (lang == NULL) ? "_NORMAL_" : gtk_source_language_get_id (lang));
 		}
 	}
 
@@ -372,6 +373,8 @@ gedit_document_class_init (GeditDocumentClass *klass)
 	buf_class->mark_set = gedit_document_mark_set;
 	buf_class->changed = gedit_document_changed;
 
+	klass->save = gedit_document_save_real;
+
 	g_object_class_install_property (object_class, PROP_URI,
 					 g_param_spec_string ("uri",
 					 		      "URI",
@@ -455,6 +458,32 @@ gedit_document_class_init (GeditDocumentClass *klass)
 			      1,
 			      G_TYPE_POINTER);
 
+	/**
+	 * GeditDocument::save:
+	 * @document: the #GeditDocument.
+	 * @uri: the uri where the document is about to be saved.
+	 * @encoding: the #GeditEncoding used to save the document.
+	 * @flags: the #GeditDocumentSaveFlags for the save operation.
+	 *
+	 * The "save" signal is emitted when the document is saved.
+	 *
+	 * Since: 2.20
+	 */
+	document_signals[SAVE] =
+		g_signal_new ("save",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GeditDocumentClass, save),
+			      NULL, NULL,
+			      gedit_marshal_VOID__STRING_BOXED_FLAGS,
+			      G_TYPE_NONE,
+			      3,
+			      G_TYPE_STRING,
+			      /* we rely on the fact that the GeditEncoding pointer stays
+			       * the same forever */
+			      GEDIT_TYPE_ENCODING | G_SIGNAL_TYPE_STATIC_SCOPE,
+			      GEDIT_TYPE_DOCUMENT_SAVE_FLAGS);
+
 	document_signals[SAVING] =
    		g_signal_new ("saving",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -508,9 +537,7 @@ set_language (GeditDocument     *doc,
 		return;
 
 	gtk_source_buffer_set_language (GTK_SOURCE_BUFFER (doc), lang);
-	if (lang != NULL)
-		gedit_language_init_tag_styles (lang);
-		
+
 	if (lang != NULL)
 		gtk_source_buffer_set_highlight (GTK_SOURCE_BUFFER (doc),
 						 gedit_prefs_manager_get_enable_syntax_highlighting ());
@@ -520,21 +547,11 @@ set_language (GeditDocument     *doc,
 
 	if (set_by_user && (doc->priv->uri != NULL))
 	{
-		gchar *lang_id = NULL;
-
-		if (lang != NULL)
-		{
-			lang_id = gtk_source_language_get_id (lang);
-			g_return_if_fail (lang_id != NULL);
-		}
-		
 		gedit_metadata_manager_set (doc->priv->uri,
-					    "language",
-					    (lang_id == NULL) ? "_NORMAL_" : lang_id);
-
-		g_free (lang_id);
+			    "language",
+			    (lang == NULL) ? "_NORMAL_" : gtk_source_language_get_id (lang));
 	}
-	
+
 	doc->priv->language_set_by_user = set_by_user;
 }
 
@@ -569,6 +586,9 @@ set_encoding (GeditDocument       *doc,
 static void
 gedit_document_init (GeditDocument *doc)
 {
+	GtkSourceStyleManager *style_manager;
+	GtkSourceStyleScheme *style_scheme;
+
 	gedit_debug (DEBUG_DOCUMENT);
 
 	doc->priv = GEDIT_DOCUMENT_GET_PRIVATE (doc);
@@ -602,6 +622,11 @@ gedit_document_init (GeditDocument *doc)
 	gedit_document_set_enable_search_highlighting (doc,
 						       gedit_prefs_manager_get_enable_search_highlighting ());
 
+	style_manager = gedit_get_source_style_manager ();
+	style_scheme = gedit_source_style_manager_get_default_scheme (style_manager);
+	gtk_source_buffer_set_style_scheme (GTK_SOURCE_BUFFER (doc),
+					    style_scheme);
+
 	g_signal_connect_after (doc, 
 			  	"insert-text",
 			  	G_CALLBACK (insert_text_cb),
@@ -698,8 +723,8 @@ set_uri (GeditDocument *doc,
 
 			if (strcmp (data, "_NORMAL_") != 0)
 			{
-				language = gedit_languages_manager_get_language_from_id (
-							gedit_get_languages_manager (),
+				language = gtk_source_language_manager_get_language_by_id (
+							gedit_get_language_manager (),
 							data);
 			}
 
@@ -711,8 +736,8 @@ set_uri (GeditDocument *doc,
 
 			if (strcmp (doc->priv->mime_type, "text/plain") != 0)
 			{
-				language = gedit_languages_manager_get_language_from_mime_type (
-							gedit_get_languages_manager (),
+				language = gedit_language_manager_get_language_from_mime_type (
+							gedit_get_language_manager (),
 							doc->priv->mime_type);
 			}
 		}
@@ -887,6 +912,40 @@ gedit_document_get_readonly (GeditDocument *doc)
 	return doc->priv->readonly;
 }
 
+gboolean
+_gedit_document_check_externally_modified (GeditDocument *doc)
+{
+	GnomeVFSFileInfo *info;
+	GnomeVFSResult result;
+	gint file_mtime;
+
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
+
+	if (doc->priv->uri == NULL)
+	{
+		return FALSE;
+	}
+
+	info = gnome_vfs_file_info_new ();
+
+	result = gnome_vfs_get_file_info_uri (doc->priv->vfs_uri,
+					      info,
+					      GNOME_VFS_FILE_INFO_DEFAULT|
+					      GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+	if (result != GNOME_VFS_OK)
+	{
+		gnome_vfs_file_info_unref (info);
+
+		return FALSE;
+	}
+
+	file_mtime = info->mtime;
+
+	gnome_vfs_file_info_unref (info);
+
+	return (file_mtime > doc->priv->mtime);
+}
+
 static void
 reset_temp_loading_data (GeditDocument       *doc)
 {
@@ -948,6 +1007,15 @@ document_loader_loaded (GeditDocumentLoader *loader,
 			gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc),
 							    &iter,
 							    MAX (offset, 0));
+
+			/* make sure it's a valid position, if the file
+			 * changed we may have ended up in the middle of
+			 * a utf8 character cluster */
+			if (!gtk_text_iter_is_cursor_position (&iter))
+			{
+				gtk_text_buffer_get_start_iter (GTK_TEXT_BUFFER (doc),
+								&iter);
+			}
 		}
 		/* otherwise to the top */
 		else
@@ -1094,7 +1162,6 @@ document_saver_saving (GeditDocumentSaver *saver,
 		/* the saver has been used, throw it away */
 		g_object_unref (doc->priv->saver);
 		doc->priv->saver = NULL;
-		doc->priv->is_saving_as = FALSE;
 	}
 	else
 	{
@@ -1115,11 +1182,10 @@ document_saver_saving (GeditDocumentSaver *saver,
 }
 
 static void
-document_save_real (GeditDocument          *doc,
-		    const gchar            *uri,
-		    const GeditEncoding    *encoding,
-		    time_t                  mtime,
-		    GeditDocumentSaveFlags  flags)
+gedit_document_save_real (GeditDocument          *doc,
+			  const gchar            *uri,
+			  const GeditEncoding    *encoding,
+			  GeditDocumentSaveFlags  flags)
 {
 	g_return_if_fail (doc->priv->saver == NULL);
 
@@ -1136,10 +1202,18 @@ document_save_real (GeditDocument          *doc,
 	gedit_document_saver_save (doc->priv->saver,
 				   uri,
 				   encoding,
-				   mtime,
+				   doc->priv->mtime,
 				   flags);
 }
 
+/**
+ * gedit_document_save:
+ * @document: the #GeditDocument.
+ * @flags: optionnal #GeditDocumentSaveFlags.
+ *
+ * Save the document to its previous location. This results in the "save"
+ * signal to be emitted.
+ */
 void
 gedit_document_save (GeditDocument          *doc,
 		     GeditDocumentSaveFlags  flags)
@@ -1147,13 +1221,24 @@ gedit_document_save (GeditDocument          *doc,
 	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 	g_return_if_fail (doc->priv->uri != NULL);
 
-	document_save_real (doc,
-			    doc->priv->uri,
-			    doc->priv->encoding,
-			    doc->priv->mtime,
-			    flags);
+	g_signal_emit (doc,
+		       document_signals[SAVE],
+		       0,
+		       doc->priv->uri,
+		       doc->priv->encoding,
+		       flags);
 }
 
+/**
+ * gedit_document_save_as:
+ * @document: the #GeditDocument.
+ * @uri: the uri where to save the document.
+ * @encoding: the #GeditEncoding to encode the document.
+ * @flags: optionnal #GeditDocumentSaveFlags.
+ *
+ * Save the document to a new location. This results in the "save" signal
+ * to be emitted.
+ */
 void
 gedit_document_save_as (GeditDocument          *doc,
 			const gchar            *uri,
@@ -1164,9 +1249,14 @@ gedit_document_save_as (GeditDocument          *doc,
 	g_return_if_fail (uri != NULL);
 	g_return_if_fail (encoding != NULL);
 
-	doc->priv->is_saving_as = TRUE;
-	
-	document_save_real (doc, uri, encoding, 0, flags);
+	/* priv->mtime refers to the the old uri (if any). Thus, it should be
+	 * ignored when saving as. */
+	g_signal_emit (doc,
+		       document_signals[SAVE],
+		       0,
+		       uri,
+		       encoding,
+		       flags | GEDIT_DOCUMENT_SAVE_IGNORE_MTIME);
 }
 
 gboolean
@@ -1202,6 +1292,18 @@ gedit_document_is_untitled (GeditDocument *doc)
 	return (doc->priv->uri == NULL);
 }
 
+gboolean
+gedit_document_is_local (GeditDocument *doc)
+{
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
+
+	if (doc->priv->uri == NULL)
+	{
+		return FALSE;
+	}
+
+	return gedit_utils_uri_has_file_scheme (doc->priv->uri);
+}
 
 gboolean
 gedit_document_get_deleted (GeditDocument *doc)
@@ -1657,16 +1759,6 @@ _gedit_document_get_seconds_since_last_save_or_load (GeditDocument *doc)
 	return (current_time.tv_sec - doc->priv->time_of_last_save_or_load.tv_sec);
 }
 
-gboolean
-_gedit_document_is_saving_as (GeditDocument *doc)
-{
-	gedit_debug (DEBUG_DOCUMENT);
-	
-	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), FALSE);
-	
-	return (doc->priv->is_saving_as);
-}
-
 static void
 search_region (GeditDocument *doc,
 	       GtkTextIter   *start,
