@@ -26,7 +26,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-prefs-manager-app.c 5403 2007-01-02 17:10:40Z sfre $
+ * $Id: gedit-prefs-manager-app.c 5645 2007-06-24 19:42:53Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -45,6 +45,7 @@
 #include "gedit-view.h"
 #include "gedit-window.h"
 #include "gedit-window-private.h"
+#include "gedit-source-style-manager.h"
 
 static void gedit_prefs_manager_editor_font_changed	(GConfClient *client,
 							 guint        cnxn_id,
@@ -111,6 +112,11 @@ static void gedit_prefs_manager_search_hl_enable_changed(GConfClient *client,
 							 GConfEntry  *entry, 
 							 gpointer     user_data);
 
+static void gedit_prefs_manager_source_style_scheme_changed
+							(GConfClient *client,
+							 guint        cnxn_id, 
+							 GConfEntry  *entry, 
+							 gpointer     user_data);
 
 static void gedit_prefs_manager_max_recents_changed	(GConfClient *client,
 							 guint        cnxn_id, 
@@ -137,6 +143,8 @@ static gint bottom_panel_size = -1;
 static gint side_panel_active_page = 0;
 static gint bottom_panel_active_page = 0;
 
+static gint active_file_filter = -1;
+
 gboolean
 gedit_prefs_manager_app_init (void)
 {
@@ -224,6 +232,11 @@ gedit_prefs_manager_app_init (void)
 				gedit_prefs_manager_search_hl_enable_changed,
 				NULL, NULL, NULL);
 
+		gconf_client_notify_add (gedit_prefs_manager->gconf_client,
+				GPM_SOURCE_STYLE_DIR,
+				gedit_prefs_manager_source_style_scheme_changed,
+				NULL, NULL, NULL);
+
 		gconf_client_notify_add (gedit_prefs_manager->gconf_client,
 				GPM_MAX_RECENTS,
 				gedit_prefs_manager_max_recents_changed,
@@ -442,6 +455,33 @@ gedit_prefs_manager_bottom_panel_active_page_can_set (void)
 	return TRUE;
 }
 
+/* File filter */
+gint
+gedit_prefs_manager_get_active_file_filter (void)
+{
+	if (active_file_filter == -1)
+		active_file_filter = gnome_config_get_int (GPM_ACTIVE_FILE_FILTER "=0");
+
+	return active_file_filter;
+}
+
+void
+gedit_prefs_manager_set_active_file_filter (gint id)
+{
+	g_return_if_fail (id >= 0);
+	
+	if (active_file_filter == id)
+		return;
+
+	active_file_filter = id;
+	gnome_config_set_int (GPM_ACTIVE_FILE_FILTER, id);
+}
+
+gboolean 
+gedit_prefs_manager_active_file_filter_can_set (void)
+{
+	return TRUE;
+}
 static void 
 gedit_prefs_manager_editor_font_changed (GConfClient *client,
 					 guint        cnxn_id, 
@@ -466,10 +506,12 @@ gedit_prefs_manager_editor_font_changed (GConfClient *client,
 		else
 			def = GPM_DEFAULT_USE_DEFAULT_FONT;
 		
-		font = NULL;
+		if (def)
+			font = gedit_prefs_manager_get_system_font ();
+		else
+			font = gedit_prefs_manager_get_editor_font ();
 	}
-	else
-	if (strcmp (entry->key, GPM_EDITOR_FONT) == 0)
+	else if (strcmp (entry->key, GPM_EDITOR_FONT) == 0)
 	{
 		if (entry->value->type == GCONF_VALUE_STRING)
 			font = g_strdup (gconf_value_get_string (entry->value));
@@ -481,11 +523,8 @@ gedit_prefs_manager_editor_font_changed (GConfClient *client,
 	else
 		return;
 
-	if ((font == NULL) && !def)
-		font = gedit_prefs_manager_get_editor_font ();
-	else
-		font = gedit_prefs_manager_get_system_font ();
-
+	g_return_if_fail (font != NULL);
+	
 	ts = gedit_prefs_manager_get_tabs_size ();
 
 	views = gedit_app_get_views (gedit_app_get_default ());
@@ -1156,6 +1195,52 @@ gedit_prefs_manager_search_hl_enable_changed (GConfClient *client,
 	}
 }
 
+static void
+gedit_prefs_manager_source_style_scheme_changed (GConfClient *client,
+						 guint        cnxn_id,
+						 GConfEntry  *entry,
+						 gpointer     user_data)
+{
+	gedit_debug (DEBUG_PREFS);
+
+	g_return_if_fail (entry->key != NULL);
+	g_return_if_fail (entry->value != NULL);
+
+	if (strcmp (entry->key, GPM_SOURCE_STYLE_SCHEME) == 0)
+	{
+		const gchar *scheme;
+		gboolean changed;
+		GtkSourceStyleScheme *style;
+		GList *docs;
+		GList *l;
+
+		if (entry->value->type == GCONF_VALUE_STRING)
+			scheme = gconf_value_get_string (entry->value);
+		else
+			scheme = GPM_DEFAULT_SOURCE_STYLE_SCHEME;
+
+		changed = _gedit_source_style_manager_set_default_scheme
+					(gedit_get_source_style_manager (),
+					 scheme);
+		if (!changed)
+			return;
+
+		style = gedit_source_style_manager_get_default_scheme
+					(gedit_get_source_style_manager ());
+
+		docs = gedit_app_get_documents (gedit_app_get_default ());
+		for (l = docs; l != NULL; l = l->next)
+		{
+			g_return_if_fail (GTK_IS_SOURCE_BUFFER (l->data));
+
+			gtk_source_buffer_set_style_scheme (GTK_SOURCE_BUFFER (l->data),
+							    style);
+		}
+
+		g_list_free (docs);
+	}
+}
+
 static void
 gedit_prefs_manager_max_recents_changed (GConfClient *client,
 					 guint        cnxn_id,
