@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-view.c,v 1.73 2006/12/07 14:50:00 paolo Exp $ 
+ * $Id: gedit-view.c 5682 2007-07-03 09:41:20Z pborelli $ 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -99,10 +99,6 @@ GtkListStore *search_completion_model = NULL;
 
 static void	gedit_view_destroy		(GtkObject       *object);
 static void	gedit_view_finalize		(GObject         *object);
-static void	gedit_view_move_cursor		(GtkTextView     *text_view,
-						 GtkMovementStep  step,
-						 gint             count,
-						 gboolean         extend_selection);
 static gint     gedit_view_focus_out		(GtkWidget       *widget,
 						 GdkEventFocus   *event);
 
@@ -156,7 +152,6 @@ gedit_view_class_init (GeditViewClass *klass)
 {
 	GObjectClass     *object_class = G_OBJECT_CLASS (klass);
 	GtkObjectClass   *gtkobject_class = GTK_OBJECT_CLASS (klass);
-	GtkTextViewClass *textview_class = GTK_TEXT_VIEW_CLASS (klass);
 	GtkWidgetClass   *widget_class = GTK_WIDGET_CLASS (klass);
 	GtkBindingSet    *binding_set;
 
@@ -169,7 +164,6 @@ gedit_view_class_init (GeditViewClass *klass)
 	klass->start_interactive_search = start_interactive_search;
 	klass->start_interactive_goto_line = start_interactive_goto_line;
 	klass->reset_searched_text = reset_searched_text;	
-	textview_class->move_cursor = gedit_view_move_cursor;
 
 	view_signals[START_INTERACTIVE_SEARCH] =
     		g_signal_new ("start_interactive_search",
@@ -213,107 +207,6 @@ gedit_view_class_init (GeditViewClass *klass)
 
 }
 
-static void
-move_cursor (GtkTextView       *text_view,
-	     const GtkTextIter *new_location,
-	     gboolean           extend_selection)
-{
-	GtkTextBuffer *buffer = text_view->buffer;
-
-	if (extend_selection)
-		gtk_text_buffer_move_mark_by_name (buffer,
-						   "insert",
-						   new_location);
-	else
-		gtk_text_buffer_place_cursor (buffer, new_location);
-
-	gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (text_view),
-				      gtk_text_buffer_get_insert (buffer),
-				      GEDIT_VIEW_SCROLL_MARGIN,
-				      FALSE,
-				      0.0,
-				      0.0);
-}
-
-/*
- * This feature is implemented in gedit and not in gtksourceview since the latter
- * has a similar feature called smart home/end that it is not compatible with this 
- * one and is more "invasive". Maybe in the future we will move this feature in 
- * gtksourceview or even better in gtktextview.
- */
-static void
-gedit_view_move_cursor (GtkTextView    *text_view,
-			GtkMovementStep step,
-			gint            count,
-			gboolean        extend_selection)
-{
-	GtkTextBuffer *buffer = text_view->buffer;
-	GtkTextMark *mark;
-	GtkTextIter cur, iter;
-
-	/* really make sure gtksourceview's home/end is disabled */
-	g_return_if_fail (!gtk_source_view_get_smart_home_end (
-						GTK_SOURCE_VIEW (text_view)));
-
-	mark = gtk_text_buffer_get_insert (buffer);
-	gtk_text_buffer_get_iter_at_mark (buffer, &cur, mark);
-	iter = cur;
-
-	if (step == GTK_MOVEMENT_DISPLAY_LINE_ENDS &&
-	    (count == -1) && gtk_text_iter_starts_line (&iter))
-	{
-		/* Find the iter of the first character on the line. */
-		while (!gtk_text_iter_ends_line (&cur))
-		{
-			gunichar c;
-
-			c = gtk_text_iter_get_char (&cur);
-			if (g_unichar_isspace (c))
-				gtk_text_iter_forward_char (&cur);
-			else
-				break;
-		}
-
-		/* if we are clearing selection, we need to move_cursor even
-		 * if we are at proper iter because selection_bound may need
-		 * to be moved */
-		if (!gtk_text_iter_equal (&cur, &iter) || !extend_selection)
-			move_cursor (text_view, &cur, extend_selection);
-	}
-	else if (step == GTK_MOVEMENT_DISPLAY_LINE_ENDS &&
-	         (count == 1) && gtk_text_iter_ends_line (&iter))
-	{
-		/* Find the iter of the last character on the line. */
-		while (!gtk_text_iter_starts_line (&cur))
-		{
-			gunichar c;
-
-			gtk_text_iter_backward_char (&cur);
-			c = gtk_text_iter_get_char (&cur);
-			if (!g_unichar_isspace (c))
-			{
-				/* We've gone one character too far. */
-				gtk_text_iter_forward_char (&cur);
-				break;
-			}
-		}
-
-		/* if we are clearing selection, we need to move_cursor even
-		 * if we are at proper iter because selection_bound may need
-		 * to be moved */
-		if (!gtk_text_iter_equal (&cur, &iter) || !extend_selection)
-			move_cursor (text_view, &cur, extend_selection);
-	}
-	else
-	{
-		/* note that we chain up to GtkTextView skipping GtkSourceView */
-		(* GTK_TEXT_VIEW_CLASS (gedit_view_parent_class)->move_cursor) (text_view,
-										step,
-										count,
-										extend_selection);
-	}
-}
-
 static gboolean
 scroll_to_cursor_on_init (GeditView *view)
 {
@@ -344,6 +237,10 @@ gedit_view_init (GeditView *view)
 
 		g_free (editor_font);
 	}
+	else
+	{
+		gedit_view_set_font (view, TRUE, NULL);
+	}
 
 	if (!gedit_prefs_manager_get_use_default_colors ())
 	{
@@ -374,8 +271,8 @@ gedit_view_init (GeditView *view)
 		      "show_margin", gedit_prefs_manager_get_display_right_margin (), 
 		      "margin", gedit_prefs_manager_get_right_margin_position (),
 		      "highlight_current_line", gedit_prefs_manager_get_highlight_current_line (), 
+		      "smart_home_end", GTK_SOURCE_SMART_HOME_END_AFTER, /* TODO: add a gconf setting */
 		      "indent_on_tab", TRUE,
-		      "smart_home_end", FALSE, /* Never changes this */
 		      NULL);
 
 	/* Make sure that the view is scrolled to the cursor so
@@ -398,6 +295,12 @@ gedit_view_destroy (GtkObject *object)
 
 	view = GEDIT_VIEW (object);
 
+	if (view->priv->scroll_idle > 0)
+	{
+		g_source_remove (view->priv->scroll_idle);
+		view->priv->scroll_idle = 0;
+	}
+
 	if (view->priv->search_window != NULL)
 	{
 		gtk_widget_destroy (view->priv->search_window);
@@ -426,9 +329,6 @@ gedit_view_finalize (GObject *object)
 	if (view->priv->tooltips != NULL)
 		g_object_unref (view->priv->tooltips);
 
-	if (view->priv->scroll_idle > 0)
-		g_source_remove (view->priv->scroll_idle);
-		
 	(* G_OBJECT_CLASS (gedit_view_parent_class)->finalize) (object);
 }
 
@@ -448,7 +348,6 @@ gedit_view_focus_out (GtkWidget *widget, GdkEventFocus *event)
 	return FALSE;
 }
 
-
 /**
  * gedit_view_new:
  * @doc: a #GeditDocument
@@ -779,36 +678,23 @@ gedit_view_set_font (GeditView   *view,
 		     gboolean     def, 
 		     const gchar *font_name)
 {
+	PangoFontDescription *font_desc = NULL;
+
 	gedit_debug (DEBUG_VIEW);
 
 	g_return_if_fail (GEDIT_IS_VIEW (view));
 
-	if (!def)
-	{
-		PangoFontDescription *font_desc = NULL;
+	if (def)
+		font_name = gedit_prefs_manager_get_system_font ();
 
-		g_return_if_fail (font_name != NULL);
-		
-		font_desc = pango_font_description_from_string (font_name);
-		g_return_if_fail (font_desc != NULL);
+	g_return_if_fail (font_name != NULL);
 
-		gtk_widget_modify_font (GTK_WIDGET (view), font_desc);
-		
-		pango_font_description_free (font_desc);		
-	}
-	else
-	{
-		GtkRcStyle *rc_style;
+	font_desc = pango_font_description_from_string (font_name);
+	g_return_if_fail (font_desc != NULL);
 
-		rc_style = gtk_widget_get_modifier_style (GTK_WIDGET (view));
+	gtk_widget_modify_font (GTK_WIDGET (view), font_desc);
 
-		if (rc_style->font_desc)
-			pango_font_description_free (rc_style->font_desc);
-
-		rc_style->font_desc = NULL;
-		
-		gtk_widget_modify_style (GTK_WIDGET (view), rc_style);
-	}
+	pango_font_description_free (font_desc);		
 }
 
 static void
@@ -1435,10 +1321,11 @@ static void
 customize_for_search_mode (GeditView *view)
 {
 	GtkWidget *icon;
-	
+
 	if (view->priv->search_mode == SEARCH)
-	{	
-		icon = gtk_image_new_from_stock (GTK_STOCK_FIND, GTK_ICON_SIZE_MENU);
+	{
+		icon = gtk_image_new_from_stock (GTK_STOCK_FIND,
+						 GTK_ICON_SIZE_MENU);
 		
 		gtk_tooltips_set_tip (view->priv->tooltips,
 				      view->priv->search_entry,
@@ -1447,18 +1334,21 @@ customize_for_search_mode (GeditView *view)
 	}
 	else
 	{
-		icon = gtk_image_new_from_stock (GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_MENU);
+		icon = gtk_image_new_from_stock (GTK_STOCK_JUMP_TO,
+						 GTK_ICON_SIZE_MENU);
 		
 		gtk_tooltips_set_tip (view->priv->tooltips,
 				      view->priv->search_entry,
 				      _("Line you want to move the cursor to"),
 				      NULL);
 	}
-	
+
 	gtk_widget_show (icon);
 	sexy_icon_entry_set_icon (SEXY_ICON_ENTRY(view->priv->search_entry),
 				  SEXY_ICON_ENTRY_PRIMARY,
 				  GTK_IMAGE (icon));
+
+	g_object_unref (icon);
 }
 
 static gboolean
