@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-taglist-plugin-parser.c 5351 2006-12-09 13:26:06Z paolo $
+ * $Id: gedit-taglist-plugin-parser.c 5677 2007-07-01 16:57:55Z pborelli $
  */
 
 /* FIXME: we should rewrite the parser to avoid using DOM */
@@ -42,14 +42,13 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
-#include <libgnome/gnome-util.h>
-
 #include <gedit/gedit-debug.h>
 
 #include "gedit-taglist-plugin-parser.h"
 
-// FIXME
-#define USER_GEDIT_TAGLIST_PLUGIN_LOCATION ".gedit-2/plugins/taglist/"
+/* we screwed up so we still look here for compatibility */
+#define USER_GEDIT_TAGLIST_PLUGIN_LOCATION_LEGACY ".gedit-2/plugins/taglist/"
+#define USER_GEDIT_TAGLIST_PLUGIN_LOCATION ".gnome2/gedit/taglist/"
 
 TagList *taglist = NULL;
 static gint taglist_ref_count = 0;
@@ -578,7 +577,8 @@ parse_taglist_dir (const gchar *dir)
 	
 	while ((e = readdir (d)) != NULL)
 	{
-		if (strncmp (e->d_name + strlen (e->d_name) - 5, ".tags", 5) == 0)
+		if (strncmp (e->d_name + strlen (e->d_name) - 5, ".tags", 5) == 0 ||
+		    strncmp (e->d_name + strlen (e->d_name) - 8, ".tags.gz", 8) == 0)
 		{
 			gchar *tags_file = g_strconcat (dir, e->d_name, NULL);
 			parse_taglist_file (tags_file);
@@ -592,7 +592,7 @@ parse_taglist_dir (const gchar *dir)
 
 TagList* create_taglist (void)
 {
-	gchar const * const home = g_get_home_dir ();
+	const gchar *home;
 
 	gedit_debug_message (DEBUG_PLUGINS, "ref_count: %d", taglist_ref_count);
 
@@ -602,14 +602,22 @@ TagList* create_taglist (void)
 		
 		return taglist;
 	}
-		
+
 	/* load user's taglists */
+	home = g_get_home_dir ();
 	if (home != NULL)
 	{
 		gchar *pdir;
 
-		pdir = gnome_util_prepend_user_home (
-				USER_GEDIT_TAGLIST_PLUGIN_LOCATION);
+		pdir = g_build_filename (home,
+					 USER_GEDIT_TAGLIST_PLUGIN_LOCATION_LEGACY,
+					 NULL);
+		parse_taglist_dir (pdir);
+		g_free (pdir);
+
+		pdir = g_build_filename (home,
+					 USER_GEDIT_TAGLIST_PLUGIN_LOCATION,
+					 NULL);
 		parse_taglist_dir (pdir);
 		g_free (pdir);
 	}
